<?php


namespace AmazingCat\ZipGate\Controller\Adminhtml\Codes;

use AmazingCat\ZipGate\Model\ResourceModel\ZipCode\CollectionFactory;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\ResponseInterface;

class Delete extends Action
{
    /**
     * @var CollectionFactory
     */
    protected $collectionFactory;

    /**
     * Delete constructor.
     * @param Context $context
     * @param CollectionFactory $collectionFactory
     */
    public function __construct(Context $context, CollectionFactory $collectionFactory)
    {
        parent::__construct($context);
        $this->collectionFactory = $collectionFactory;
    }

    /**
     * Execute action based on request and return result
     *
     * Note: Request will be added as operation argument in future
     *
     * @return \Magento\Framework\Controller\ResultInterface|ResponseInterface
     */
    public function execute()
    {
        $excluded = $this->getRequest()->getParam('excluded', []);
        $selected = $this->getRequest()->getParam('selected', []);
        $itemsToDelete = [];

        /** @var \AmazingCat\ZipGate\Model\ResourceModel\ZipCode\Collection $collection */
        $collection = $this->collectionFactory->create();

        if ($excluded === false) { // remove all zips
            $itemsToDelete = $collection->getItems();
        } elseif (!empty($excluded)) {
            $itemsToDelete = $collection
                ->addFieldToFilter('id', ['nin' => $excluded])
                ->getItems();
        } elseif (!empty($selected)) {
            $itemsToDelete = $collection
                ->addFieldToFilter('id', ['in' => $selected])
                ->getItems();
        } else {
            $id = $this->getRequest()->getParam('id');
            $itemsToDelete[] = $collection
                ->addFieldToFilter('id', $id)
                ->getFirstItem();
        }

        $resultRedirect = $this->resultRedirectFactory->create();

        if (!empty($itemsToDelete)) {
            try {
                foreach ($itemsToDelete as $itemToDelete) {
                    $itemToDelete->delete();
                }

                $this->messageManager->addSuccessMessage(__('Zip has been deleted! (Count: ' . count($itemsToDelete) . ')'));

                return $resultRedirect->setPath('*/*/index');
            } catch (\Exception $ex) {
                $this->messageManager->addSuccessMessage($ex->getMessage());

                return $resultRedirect->setPath('*/*/index');
            }
        }

        $this->messageManager->addErrorMessage(__('Zip does not exist'));

        return $resultRedirect->setPath('*/*/');
    }
}
