<?php


namespace AmazingCat\ZipGate\Controller\Adminhtml\Codes;

use AmazingCat\ZipGate\Model\ResourceModel\ZipCode;
use AmazingCat\ZipGate\Model\ZipCodeFactory;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\App\State;

class Save extends Action implements HttpPostActionInterface
{
    /**
     * @var ZipCodeFactory
     */
    protected $zipCodeFactory;

    /**
     * @var ZipCode
     */
    protected $zipCode;

    /**
     * @var \Magento\Framework\App\State
     */
    private $appState;

    /**
     * Save constructor.
     * @param Context $context
     * @param ZipCodeFactory $zipCodeFactory
     * @param ZipCode $zipCode
     * @param \Magento\Framework\App\State $appState
     */
    public function __construct(
        Context $context,
        ZipCodeFactory $zipCodeFactory,
        ZipCode $zipCode,
        \Magento\Framework\App\State $appState
    ) {
        parent::__construct($context);
        $this->zipCodeFactory = $zipCodeFactory;
        $this->zipCode = $zipCode;
        $this->appState = $appState;
    }

    /**
     * Execute action based on request and return result
     *
     * Note: Request will be added as operation argument in future
     *
     * @return \Magento\Framework\Controller\ResultInterface|ResponseInterface
     * @throws \Magento\Framework\Exception\NotFoundException
     */
    public function execute()
    {
        /*return $this->resultFactory->create(ResultFactory::TYPE_PAGE);*/
        $data = $this->getRequest()->getParams();

        /** @var \AmazingCat\ZipGate\Model\ZipCode $zipCodeDto */
        $zipCodeDto = $this->zipCodeFactory->create();
        $zipCodeDto->setZipCode($data['zip_code'])->setStoreviewId($data['storeview_id']);

        $resultRedirect = $this->resultRedirectFactory->create();
        try {
            $this->zipCode->save($zipCodeDto);

            $this->messageManager->addSuccessMessage(__('Item saved successfully'));
            return $resultRedirect->setPath('*/codes/index');
        } catch (\Throwable $e) {
            $this->messageManager->addErrorMessage(__('There has been an error while processing your request'));

            // Add message to developer whenever there is an error
            if ($this->appState->getMode() == State::MODE_DEVELOPER) {
                $this->messageManager->addErrorMessage(
                    __("You're seeing this message because Magento are set to developer mode: ") . $e->getMessage()
                );
            }

            return $resultRedirect->setRefererUrl();
        }
    }
}
