<?php


namespace AmazingCat\ZipGate\Controller\Code;

use AmazingCat\ZipGate\Helper\ZipCodeHelper;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Exception\NotFoundException;
use Magento\Store\Model\StoreIsInactiveException;
use Zend\Log\Logger;

class Apply extends Action implements HttpPostActionInterface
{
    /**
     * @var ZipCodeHelper
     */
    protected $zipCodeHelper;

    /**
     * @var Logger
     */
    protected $logger;

    /**
     * Apply constructor.
     * @param Context $context
     * @param ZipCodeHelper $zipCodeHelper
     * @param Logger $logger
     */
    public function __construct(
        Context $context,
        ZipCodeHelper $zipCodeHelper,
        Logger $logger
    ) {
        parent::__construct($context);
        $this->zipCodeHelper = $zipCodeHelper;
        $this->logger = $logger;
    }

    /**
     * Execute action based on request and return result
     *
     * Note: Request will be added as operation argument in future
     *
     * @return \Magento\Framework\Controller\ResultInterface|ResponseInterface
     * @throws \Exception
     */
    public function execute()
    {
        $zipCode = $this->getRequest()->getParam('zipcode');
        try {
            if ($zipCode == null) {
                throw new \Exception(__('Zip Code parameter are not set in the request body'));
            }

            $store = $this->zipCodeHelper->setStoreViewByCode($zipCode);
            $storeUrl = $store->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_WEB);

            $this->messageManager->addSuccessMessage(__('Store was changed according to your zip code'));
            $eventStatus = [
                'is_success' => true,
                'message' => __('Store changed successfully')
            ];

        } catch (NotFoundException $e) {
            $errorMessage = __('We doesn\'t available in your area yet.');
            $this->messageManager->addErrorMessage($errorMessage);

            $eventStatus = [
                'is_success' => false,
                'message' => $errorMessage,
            ];
        } catch (StoreIsInactiveException $e) {
            $errorMessage = __('We currently aren\'t available in this area.');
            $this->messageManager->addErrorMessage($errorMessage);

            $eventStatus = [
                'is_success' => false,
                'message' => $errorMessage,
            ];
        } catch (\Throwable $e) {
            $this->logger->alert('ZipGate Error: ' . $e->getMessage());
            $errorMessage = $e->getMessage();
            $this->messageManager->addErrorMessage(__('Something unusual happened. Please, try again later'));

            $eventStatus = [
                'is_success' => false,
                'message' => $errorMessage,
            ];
        }
        $eventStatus['zip_code'] = $zipCode;
        $this->_eventManager->dispatch('zipgate_code_apply_postdispatch', $eventStatus);
        return !isset($storeUrl) ?
            $this->resultRedirectFactory->create()->setRefererUrl() :
            $this->resultRedirectFactory->create()->setUrl($storeUrl);
    }
}
