<?php


namespace AmazingCat\ZipGate\Setup;

use Magento\Backend\Block\Widget\Tab;
use Magento\Framework\DB\Adapter\AdapterInterface;
use Magento\Framework\DB\Ddl\Table;
use Magento\Framework\Setup\InstallSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;

class InstallSchema implements InstallSchemaInterface
{

    /**
     * Installs DB schema for a module
     *
     * @param SchemaSetupInterface $setup
     * @param ModuleContextInterface $context
     * @return void
     * @throws \Zend_Db_Exception
     */
    public function install(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        $installer = $setup;
        $installer->startSetup();
        $connection = $installer->getConnection();
        $this->createZipcodesTable($installer, $connection);
        $this->createLogTable($installer, $connection);
    }

    /**
     * @param SchemaSetupInterface $installer
     * @param AdapterInterface $connection
     * @throws \Zend_Db_Exception
     */
    protected function createZipcodesTable(SchemaSetupInterface $installer, AdapterInterface $connection)
    {
        $table = $connection->newTable(
            $installer->getTable('zipgate_codes')
        )
            ->addColumn(
                'id',
                Table::TYPE_INTEGER,
                null,
                [
                    'nullable' => false,
                    'unsigned' => true,
                    'identity' => true,
                    'primary' => true
                ],
                'ID'
            )
            ->addColumn(
                'zip_code',
                Table::TYPE_INTEGER,
                null,
                [
                    'nullable' => false,
                    'unsigned' => true
                ],
                'Zip code'
            )
            ->addColumn(
                'storeview_id',
                Table::TYPE_SMALLINT,
                null,
                [
                    'identity' => false,
                    'nullable' => false,
                    'primary' => false,
                    'unsigned' => true,
                ]
            )
            ->addForeignKey(
                $installer->getFkName('zipgate_codes', 'storeview_id', 'store', 'store_id'),
                'storeview_id',
                'store',
                'store_id'

            );
        $connection->createTable($table);

        $connection->addIndex(
            $installer->getTable('zipgate_codes'),
            $installer->getIdxName(
                $installer->getTable('zipgate_codes'),
                [
                    'zip_code',
                ],
                AdapterInterface::INDEX_TYPE_INDEX
            ),
            ['zip_code'],
            AdapterInterface::INDEX_TYPE_INDEX
        );
    }

    /**
     * @param SchemaSetupInterface $installer
     * @param AdapterInterface $connection
     * @throws \Zend_Db_Exception
     */
    protected function createLogTable(SchemaSetupInterface $installer, AdapterInterface $connection)
    {
        $table = $connection->newTable(
            $installer->getTable('zipgate_log')
        )
            ->addColumn(
                'id',
                Table::TYPE_INTEGER,
                null,
                [
                    'nullable' => false,
                    'unsigned' => true,
                    'primary' => true,
                    'identity' => true
                ],
                'ID'
            )
            ->addColumn(
                'zip_code',
                Table::TYPE_INTEGER,
                null,
                [
                    'nullable' => false,
                    'unsigned' => true,
                    'primary' => false,
                    'identity' => false
                ],
                'Zip Code'
            )
            ->addColumn(
                'status',
                Table::TYPE_TEXT,
                255,
                [
                    'nullable' => false,
                ],
                'Zip Code Passed Status'
            )
            ->addColumn(
                'message',
                Table::TYPE_TEXT,
                255,
                [
                    'nullable' => false
                ],
                'Log Message'
            )
            ->addColumn(
                'created_at',
                Table::TYPE_DATETIME,
                null,
                [
                    'nullable' => false,
                ],
                'Code Applied At Time'
            )
            ->addIndex(
                $connection->getIndexName(
                    $installer->getTable('zipgate_log'),
                    ['status', 'message'],
                    AdapterInterface::INDEX_TYPE_FULLTEXT
                ),
                ['status', 'message']
            );

        $connection->createTable($table);
    }
}
